import { Injectable } from "@angular/core";
import {
  HttpClient,
  HttpHeaders,
  HttpParams,
} from "@angular/common/http";
import { Observable } from "rxjs";
import { environment } from "../../../environments/environment";

@Injectable({
  providedIn: "root"
})
export class ApiBaseService {
  baseUrl = environment.apiBaseURL;

  constructor(private httpClient: HttpClient) { }

  public getFullApiRoute(route: string, id?: number): string {
    let apiRoute;
    if (id) {
      apiRoute = `${this.baseUrl}/${route}/${id}`;
    } else {
      apiRoute = `${this.baseUrl}/${route}/`;
    }
    return apiRoute;
  }

  private getDefaultHeaders() {
    let headers = new HttpHeaders();
    headers = headers
      .append("Content-Type", "application/json")
      .append("Access-Control-Allow-Methods", "*")
      .append("Access-Control-Allow-Methods", "GET,POST,OPTIONS,DELETE,PUT")
      .append("Access-Control-Allow-Origin", "*")

    return headers;
  }

  private getHttpParams(params?) {
    return new HttpParams({
      fromObject: params

    });
  }

  private getOptions(optionalParams?) {
    const headers = this.getDefaultHeaders();
    const httpParams = this.getHttpParams(optionalParams);
    return { params: httpParams, headers: headers };
  }

  /**
   * GET
   */
  public get<T>(route: string, params?): Observable<T> {
    return this.httpClient.get<T>(
      this.getFullApiRoute(route),
      this.getOptions(params)
    );
  }

  public getById<T>(route: string, id: number, params?): Observable<T> {
    return this.httpClient.get<T>(
      this.getFullApiRoute(route, id),
      this.getOptions(params)
    );
  }

  /**
   * POST
   */
  public post<T>(route: string, data?, params?): Observable<T> {
    return this.httpClient.post<T>(
      this.getFullApiRoute(route),
      JSON.stringify(data),
      this.getOptions(params)
    );
  }

  public postById<T>(route: string, id: number, data?, params?): Observable<T> {
    return this.httpClient.post<T>(
      this.getFullApiRoute(route, id),
      data,
      this.getOptions(params)
    );
  }


  /**
   * PUT
   */
  public put(route: string, data, params?) {
    return this.httpClient.put(
      this.getFullApiRoute(route),
      JSON.stringify(data),
      this.getOptions(params)
    );
  }

  public putById(route: string, id: number, data?, params?) {
    return this.httpClient.put(
      this.getFullApiRoute(route, id),
      data,
      this.getOptions(params)
    );
  }

  /**
   * DELETE
   */
  public delete(route: string, params?) {
    return this.httpClient.delete(
      this.getFullApiRoute(route),
      this.getOptions(params)
    );
  }

  public deleteById(route: string, id: number, params?) {
    return this.httpClient.delete(
      this.getFullApiRoute(route, id),
      this.getOptions(params)
    );
  }
}
